package func_test

import (
	"fmt"
	"math/rand"
	"testing"
	"time"
)

// 多返回值函数
func returnMultiValues() (int, int) {
	return rand.Intn(10), rand.Intn(20)
}

// 函数用法之一: 统计执行时间
func timeSpent(inner func(op int) int) func(op int) int {
	return func(n int) int {
		start := time.Now()
		ret := inner(n)
		fmt.Println("time spent: ", time.Since(start).Seconds())
		return ret
	}
}

func slowFun(op int) int {
	time.Sleep(time.Second * 1)
	return op
}

func TestFn(t *testing.T) {
	a, b := returnMultiValues()
	t.Log(a, b)

	tsSF := timeSpent(slowFun)
	t.Log(tsSF(10))
}

// 求和
func Sum(ops ...int) int {
	ret := 0
	for _, op := range ops {
		ret += op
	}
	return ret
}

func TestVarParam(t *testing.T) {
	t.Log(Sum(1, 3, 5, 7, 9, 2, 6))
}

func Clear() {
	fmt.Println("Clear resources")
}

// defer 延迟执行 一般用于资源的释放和异常的捕捉
func TestDefer(t *testing.T) {
	defer Clear()
	fmt.Println("Start")
	panic("program error") //抛出异常
}
