package error_test

import (
	"errors"
	"fmt"
	"testing"
)

/*

	Go的错误机制
	1.没有异常机制
	2.error类型实现了error接口
	3.可以通过 errors.New 来快速创建错误实例

*/

var ErrorLessThanTwo = errors.New("n should be not less than 2")
var ErrorLargerThenHundred = errors.New("n should be not larger than 100")

func GetFibonacci(n int) ([]int, error) {
	if n < 2 {
		return nil, ErrorLessThanTwo
	}
	if n > 100 {
		return nil, ErrorLargerThenHundred
	}
	fibList := []int{1, 1}
	for i := 2; i < n; i++ {
		fibList = append(fibList, fibList[i-2]+fibList[i-1])
	}
	return fibList, nil
}

func TestGetFibonacci(t *testing.T) {
	if ret, err := GetFibonacci(-10); err != nil {
		if err == ErrorLessThanTwo {
			fmt.Println("It is less.")
		}
		t.Error(err)
	} else {
		t.Log(ret)
	}

}
