package channel_close_test

import (
	"fmt"
	"sync"
	"testing"
)

// 生产者
func dataProducer(ch chan int, wg *sync.WaitGroup) {
	go func() {
		for i := 0; i < 10; i++ {
			ch <- i
		}
		close(ch) // 关闭channel
		wg.Done()
	}()
}

// 消费者
// func dataReceiver(ch chan int, wg *sync.WaitGroup) {
// 	go func() {
// 		for i := 0; i < 10; i++ {
// 			data := <-ch
// 			fmt.Println(data)
// 		}
// 		wg.Done()
// 	}()
// }

func dataReceiver(ch chan int, wg *sync.WaitGroup) {
	go func() {
		for {
			if data, ok := <-ch; ok {
				fmt.Println(data)
			} else {
				// 通道关闭 去获取值会立即返回通道定义类型的默认值
				fmt.Println(data)
				break
			}
		}
		wg.Done()
	}()
}

func TestCloseChannel(t *testing.T) {
	var wg sync.WaitGroup
	ch := make(chan int)
	wg.Add(1)
	dataProducer(ch, &wg)
	wg.Add(1)
	dataReceiver(ch, &wg)
	wg.Add(1)
	dataReceiver(ch, &wg)
	wg.Wait()
}
