package obj_pool

import (
	"errors"
	"time"
)

type MyObj struct {
}

type ObjPool struct {
	buffChan chan *MyObj
}

// 利用channel实现的对象池

func NewObjPool(num int) *ObjPool {
	objPool := ObjPool{}
	objPool.buffChan = make(chan *MyObj, num)
	for i := 0; i < num; i++ {
		objPool.buffChan <- &MyObj{}
	}
	return &objPool
}

func (p *ObjPool) GetObj(timeout time.Duration) (*MyObj, error) {
	select {
	case ret := <-p.buffChan:
		return ret, nil
	case <-time.After(timeout): // 超时控制
		return nil, errors.New("超时")
	}
}

func (p *ObjPool) PutObj(obj *MyObj) error {
	select {
	case p.buffChan <- obj:
		return nil
	default:
		return errors.New("超出对象池最大容量")
	}
}
