package obj_cache

import (
	"runtime"
	"sync"
	"testing"
)

func TestSyncPool(t *testing.T) {
	pool := &sync.Pool{
		New: func() interface{} {
			t.Log("Create a new object.")
			return 100
		},
	}

	v := pool.Get().(int)
	t.Log(v)
	pool.Put(3)

	v1 := pool.Get().(int)
	t.Log("put操作后: ", v1)
	runtime.GC() // GC 会清除sync.pool中缓存的对象
	v2, _ := pool.Get().(int)
	t.Log("GC操作后: ", v2)

}

// 协程测试
func TestSyncPoolInMultiGroutine(t *testing.T) {
	pool := &sync.Pool{
		New: func() interface{} {
			t.Log("create object")
			return 10
		},
	}

	pool.Put(100)
	pool.Put(100)
	pool.Put(100)

	var wg sync.WaitGroup
	for i := 0; i < 10; i++ {
		wg.Add(1)
		go func(id int) {
			t.Log("从pool获取对象: ", pool.Get())
			wg.Done()
		}(i)
	}
	wg.Wait()
}
